@@ -24,9 +24,11 @@ class NoStateTransitionsDefined(ObjectException):
     pass
 
 
+class MultipleObjects(ObjectException):
+    pass
+
+
 # Instance
-
-
 class InstanceException(Exception):
     pass
 
@@ -39,6 +41,11 @@ class InstanceBadDiskSpecification(InstanceException):
     pass
 
 
+class NVRAMTemplateMissing(InstanceException):
+    pass
+
+
+# Scheduler
 class SchedulerException(Exception):
     pass
 
@@ -76,6 +83,19 @@ class BadObjectVersion(DatabaseException):
     pass
 
 
+class PreExistingReadOnlyCache(DatabaseException):
+    pass
+
+
+class ForbiddenWhileUsingReadOnlyCache(DatabaseException):
+    pass
+
+
+class PrefixNotInCache(DatabaseException):
+    pass
+
+
+# Virt
 class VirtException(Exception):
     pass
 
@@ -84,6 +104,7 @@ class NoDomainException(VirtException):
     pass
 
 
+# Config
 class FlagException(Exception):
     pass
 
@@ -97,6 +118,10 @@ class ImagesCannotShrinkException(Exception):
     pass
 
 
+class ImageMissingFromCache(Exception):
+    pass
+
+
 # Tasks
 class TaskException(Exception):
     pass
@@ -177,9 +202,9 @@ class BlobMissing(ArtifactException):
     pass
 
 
-class BlobRefCountDecrementBelowZero(ArtifactException):
-    pass
-
-
 class BlobDeleted(ArtifactException):
     pass
+
+
+class BlobFetchFailed(Exception):
+    pass
