@@ -1,11 +1,13 @@
-import copy
 from flask_jwt_extended import jwt_required
 from functools import partial
+import os
+import shutil
+import time
 import uuid
 
 from shakenfist import artifact
 from shakenfist.artifact import Artifact, Artifacts
-from shakenfist.blob import Blob
+from shakenfist import blob
 from shakenfist.config import config
 from shakenfist.daemons import daemon
 from shakenfist import etcd
@@ -19,13 +21,20 @@ daemon.set_log_level(LOG, 'api')
 
 class InstanceSnapshotEndpoint(api_base.Resource):
     @jwt_required
-    @api_base.arg_is_instance_uuid
+    @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     @api_base.redirect_instance_request
     @api_base.requires_instance_active
-    def post(self, instance_uuid=None, instance_from_db=None, all=None,
-             device=None):
+    def post(self, instance_ref=None, instance_from_db=None, all=None,
+             device=None, max_versions=0):
         disks = instance_from_db.block_devices['devices']
+        if instance_from_db.uefi:
+            disks.append({
+                'type': 'nvram',
+                'device': 'nvram',
+                'path': os.path.join(instance_from_db.instance_path, 'nvram'),
+                'snapshot_ignores': False
+            })
 
         # Filter if requested
         if device:
@@ -36,18 +45,26 @@ class InstanceSnapshotEndpoint(api_base.Resource):
             disks = new_disks
         elif not all:
             disks = [disks[0]]
+        LOG.with_fields({
+            'instance': instance_from_db.uuid,
+            'devices': disks}).info('Devices for snapshot')
 
         out = {}
         for disk in disks:
             if disk['snapshot_ignores']:
                 continue
 
-            if disk['type'] != 'qcow2':
+            if disk['type'] not in ['qcow2', 'nvram']:
+                continue
+
+            if not os.path.exists(disk['path']):
                 continue
 
             a = Artifact.from_url(
                 Artifact.TYPE_SNAPSHOT,
-                '%s%s/%s' % (artifact.INSTANCE_URL, instance_uuid, disk['device']))
+                '%s%s/%s' % (artifact.INSTANCE_URL,
+                             instance_from_db.uuid, disk['device']),
+                max_versions)
 
             blob_uuid = str(uuid.uuid4())
             entry = a.add_index(blob_uuid)
@@ -59,24 +76,41 @@ class InstanceSnapshotEndpoint(api_base.Resource):
                 'blob_uuid': blob_uuid
             }
 
-            etcd.enqueue(config.NODE_NAME, {
-                'tasks': [SnapshotTask(instance_uuid, disk, a.uuid, blob_uuid)],
-            })
-            instance_from_db.add_event('api', 'snapshot of %s requested' % disk,
-                                       None, a.uuid)
+            if disk['type'] == 'nvram':
+                # These are small and don't use qemu-img to capture, so just
+                # do them now.
+                blob.ensure_blob_path()
+                dest_path = blob.Blob.filepath(blob_uuid)
+                shutil.copyfile(disk['path'], dest_path)
+
+                st = os.stat(dest_path)
+                b = blob.Blob.new(blob_uuid, st.st_size,
+                                  time.time(), time.time())
+                b.observe()
+                a.state = Artifact.STATE_CREATED
+
+            else:
+                etcd.enqueue(config.NODE_NAME, {
+                    'tasks': [SnapshotTask(
+                        instance_from_db.uuid, disk, a.uuid, blob_uuid)],
+                })
+            instance_from_db.add_event(
+                'api', 'snapshot of %s requested' % disk['path'].split('/')[-1],
+                None, a.uuid)
 
         return out
 
     @jwt_required
-    @api_base.arg_is_instance_uuid
+    @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
-    def get(self, instance_uuid=None, instance_from_db=None):
+    def get(self, instance_ref=None, instance_from_db=None):
         out = []
-        for snap in Artifacts([partial(artifact.instance_snapshot_filter, instance_uuid)]):
+        for snap in Artifacts([
+                partial(artifact.instance_snapshot_filter, instance_from_db.uuid)]):
             ev = snap.external_view_without_index()
             for idx in snap.get_all_indexes():
                 # Give the blob uuid a better name
-                b = Blob.from_db(idx['blob_uuid'])
+                b = blob.Blob.from_db(idx['blob_uuid'])
                 if not b:
                     continue
 
@@ -85,7 +119,7 @@ class InstanceSnapshotEndpoint(api_base.Resource):
                 del bout['uuid']
 
                 # Merge it with the parent artifact
-                a = copy.copy(ev)
+                a = ev.copy()
                 a.update(bout)
                 out.append(a)
         return out
